﻿namespace SddpViewer;

using Avalonia.Threading;

using System.Collections.ObjectModel;
using System.Threading.Tasks;

using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;

using Rssdp;

public sealed partial class MainWindowViewModel : ObservableObject, IDisposable
{
    public MainWindowViewModel()
    {
        NetworkAdapters = NetworkAdapter.GetAvailableNetworkAdapter().ToArray();
        SelectedNetworkAdapter = NetworkAdapters.FirstOrDefault();
    }

    private void LocatorOnDeviceUnavailable(object? sender, DeviceUnavailableEventArgs e)
    {
        var existingDevice = SddpDevices.FirstOrDefault(
            x => string.Equals(x.Usn, e.DiscoveredDevice.Usn, StringComparison.Ordinal)
        );
        if (existingDevice is null)
            return;
        Dispatcher.UIThread.Invoke(() => SddpDevices.Remove(existingDevice));
    }

    private async void LocatorOnDeviceAvailable(object? sender, DeviceAvailableEventArgs e)
    {
        if (!e.IsNewlyDiscovered)
        {
            return;
        }

        var discoveredDeviceViewModel = new DiscoveredDeviceViewModel(e.DiscoveredDevice);
        Dispatcher.UIThread.Invoke(() => SddpDevices.Add(discoveredDeviceViewModel));
        await discoveredDeviceViewModel.GetFurtherInformationAsync();
    }

    [ObservableProperty]
    private IReadOnlyList<NetworkAdapter> _networkAdapters;

    [ObservableProperty]
    private NetworkAdapter? _selectedNetworkAdapter;

    [ObservableProperty]
    private string _deviceIpAddress = "192.168.42.193";

    [ObservableProperty]
    private string _notificationFilter = "upnp:rootdevice";

    private SsdpDeviceLocator? _locator;

    public ObservableCollection<DiscoveredDeviceViewModel> SddpDevices { get; } = new();

    [RelayCommand]
    private async Task SearchDevicesNowAsync()
    {
        SddpDevices.Clear();
        if (_locator is not null)
        {
            _locator.StopListeningForNotifications();
            _locator.DeviceAvailable -= LocatorOnDeviceAvailable;
            _locator.DeviceUnavailable -= LocatorOnDeviceUnavailable;
        }
        _locator = new SsdpDeviceLocator(SelectedNetworkAdapter?.IpAddress.ToString());
        if (!string.IsNullOrWhiteSpace(NotificationFilter))
        {
            _locator.NotificationFilter = NotificationFilter;
        }

        _locator.DeviceAvailable += LocatorOnDeviceAvailable;
        _locator.DeviceUnavailable += LocatorOnDeviceUnavailable;
        _locator.StartListeningForNotifications();
        await _locator.SearchAsync();
    }

    public void Dispose()
    {
        _locator?.Dispose();
    }
}
